#ifndef KDTREE_H
#define KDTREE_H

#include <qglobal.h>
#include "vec.h"
#include <unordered_set>

struct Triangle
{
    quint32 a, b, c;
};

class KdTree
{
public:
    KdTree(const Vec3 *vertex, Triangle *begin, Triangle *end);
    ~KdTree();

    template<typename F>
    void queryZ(const double z, F fn) const
    {
        if (begin == end)
            return;

        if (center.z() - radius > z
                || center.z() + radius < z)
            return;

        if (!childs)
        {
            for(const Triangle *t = begin ; t < end ; ++t)
            {
                const double minZ = std::min(vertex[t->a].z(), std::min(vertex[t->b].z(), vertex[t->c].z()));
                const double maxZ = std::max(vertex[t->a].z(), std::max(vertex[t->b].z(), vertex[t->c].z()));
                if (minZ <= z && maxZ >= z)
                    fn(*t);
            }
            return;
        }
        for(size_t i = 0 ; i < 3 ; ++i)
            childs[i].queryZ(z, fn);
    }

	template<typename F>
	void queryZ(const double z_min, const double z_max, F fn) const
	{
		if (begin == end)
			return;

		if (center.z() - radius > z_max
				|| center.z() + radius < z_min)
			return;

		if (!childs)
		{
			for(const Triangle *t = begin ; t < end ; ++t)
			{
				const double minZ = std::min(vertex[t->a].z(), std::min(vertex[t->b].z(), vertex[t->c].z()));
				const double maxZ = std::max(vertex[t->a].z(), std::max(vertex[t->b].z(), vertex[t->c].z()));
				if (minZ <= z_max && maxZ >= z_min)
					fn(*t);
			}
			return;
		}
		for(size_t i = 0 ; i < 3 ; ++i)
			childs[i].queryZ(z_min, z_max, fn);
	}

    const Triangle *queryRay(const Vec3 &pos,
                             const Vec3 &dir,
                             Vec3 &hit_point,
                             const std::unordered_set<const Triangle *> &avoid = std::unordered_set<const Triangle *>(),
                             double max_dist = std::numeric_limits<double>::infinity()) const;

private:
    KdTree();
    void build(const Vec3 *vertex, Triangle *begin, Triangle *end);
	inline bool rayTriangleCollision(const Vec3 &pos, const Vec3 &dir, const Vec3 &a, const Vec3 &b, const Vec3 &c, Vec3 &p) const;

private:
    Vec3 center;
    double radius;
    KdTree *childs;
    Triangle *begin;
    Triangle *end;
    const Vec3 *vertex;
    int n;
};

#endif // KDTREE_H
