#ifndef PRINTSETTINGS_H
#define PRINTSETTINGS_H

#include <QDockWidget>
#include <map>
#include <deque>
#include "vec.h"

namespace Ui {
class PrintSettings;
}

class PrintSettings : public QDockWidget
{
    Q_OBJECT
    
public:
    explicit PrintSettings(QWidget *parent = 0);
    ~PrintSettings();
    
    double getLayerHeight() const;
    double getExtruderTargetTemperature() const;
    double getBedTargetTemperature() const;
    double getExtrusionSpeed() const;
    double getSupportDensityRatio() const;
    double getExtrusionSpeedAtStart() const;
	bool isSupportEnabled() const;
	bool isMinimalSupportEnabled() const;
    bool isZmoveEnabled() const;
    double getFillFactor() const;
    double getExtrudedWidth() const;
    double getZOffset() const;
    double getExtrusionMultiplier() const;
	double getRetractionOnMove() const;
    double getXmax() const;
    double getYmax() const;
    double getZmax() const;
    double getShellOffset() const;
    int getNbShells() const;
    double getXscale() const;
    double getYscale() const;
    double getZscale() const;

public slots:
	void resetTransform();

private slots:
    void loadSettings();
    void saveSettings();
    void addTemperatureSample(double extruder_temperature, double bed_temperature);
    void toggleHeat(bool b);
    void handleGo();
    void handleUpdateAxisPos();
    void handleHomeAll();
    void setAxisPos(double X, double Y, double Z);
    void setAxisPosX(double X);
    void setAxisPosY(double Y);
    void setAxisPosZ(double Z);
    void updateTransform();
	void autoCenter();
    void updateBuildRange() const;

private:
    Ui::PrintSettings *ui;
    std::deque<std::pair<double, double> > extruder_temperature_samples;
    std::deque<std::pair<double, double> > bed_temperature_samples;
    bool b_activate_extruder_notification;
	Vec3 mesh_center;

private:
    static PrintSettings *pInstance;
public:
    static PrintSettings *instance();
};

#endif // PRINTSETTINGS_H
